function [px]=earth_asteroid_model(t,x,P)
% This routine contains the dynamics of the earth and an asteroid
% Note it is assumed that the mass of the earth and asteroid are 
% negligible compare to the sun
% 
% [px] = earth_asteroid_model(t,x,P);
%
% Inputs:
% t        = time {not actually used} (s)
% x        = state vector
%  x(1)    = x-component of earth velocity (m/s)
%  x(2)    = y-component of earth velocity (m/s)
%  x(3)    = x-component of earth position (m)
%  x(4)    = y-component of earth position (m)
%  x(5)    = x-component of asteroid velocity (m/s)
%  x(6)    = y-component of asteroid velocity (m/s)
%  x(7)    = x-component of asteroid position (m)
%  x(8)    = y-component of asteroid position (m)
% P        = structure with parameters
%  P.ms    = mass of sun (kg)
%  P.me    = mass of earth (kg)
%  P.ma    = mass of asteroid (kg)
%  P.G     = gravitational constant (N*(m/kg)^2)
%  P.evx0  = x-component of initial earth velocity(m/s)
%  P.evy0  = y-component of initial earth velocity(m/s)
%  P.epx0  = x-component of initial earth position(m)
%  P.epy0  = y-component of initial earth position(m)
%  P.avx0  = x-component of initial asteroid velocity(m/s)
%  P.avy0  = y-component of initial asteroid velocity(m/s)
%  P.apx0  = x-component of initial asteroid position(m)
%  P.apy0  = y-component of initial asteroid position(m)
% 
% Outputs:
% px       = time derivative of state vector (see defintion of x)
%
% Internal:
% evx      = x-component of earth velocity (m/s)
% evy      = y-component of earth velocity (m/s)
% epx      = x-component of earth position (m)
% epy      = y-component of earth position (m)
% avx      = x-component of asteroid velocity (m/s)
% avy      = y-component of asteroid velocity (m/s)
% apx      = x-component of asteroid position (m)
% apy      = y-component of asteroid position (m)
% res2     = square of earth-sun distance (m^2)
% res      = earth-sun distance (m)
% fes      = magnitude of earth-sun force (N)
% fesx     = sun's gravitation force on earth in x-direction (N)
% fesy     = sun's gravitation force on earth in y-direction (N)
% ras2     = square of asteroid-sun distance (m^2)
% ras      = asteroid-sun distance (m)
% fas      = magnitude of asteroid-sun force (N)
% fasx     = sun's gravitation force on asteroid in x-direction (N)
% fasy     = sun's gravitation force on asteroid in y-direction (N)
% Dx       = distance from earth to asteroid in x-direction (m)
% Dy       = distance from earth to asteroid in y-direction (m)
% rae2     = square of asteroid-earth distance (m^2)
% rae      = asteroid-earth distance (m)
% fae      = magnitude of asteroid-earth force (N)
% faex     = earth's gravitation force on asteroid in x-direction (N)
% faey     = earth's gravitation force on asteroid in y-direction (N)
% pevx     = time derivative of x-component of earth velocity (m/s^2)
% pevy     = time derivative of y-component of earth velocity (m/s^2)
% pepx     = time derivative of x-component of earth position (m/s)
% pepy     = time derivative of y-component of earth position (m/s)
% pavx     = time derivative of x-component of asteroid velocity (m/s^2)
% pavy     = time derivative of y-component of asteroid velocity (m/s^2)
% papx     = time derivative of x-component of asteroid position (m/s)
% papy     = time derivative of y-component of asteroid position (m/s)
%
% Written by S.D. Sudhoff 
%   School of Electrical and Computer Engineering
%   1285 Electrical Engineering Building
%   West Lafayette, IN 47907-1285
%   Phone: 765-494-3246
%   Fax: 765-494-0676
%   E-mail: sudhoff@ecn.purdue.edu

% decompose state vector
evx = x(1);
evy = x(2);
epx = x(3);
epy = x(4);
avx = x(5);
avy = x(6);
apx = x(7); 
apy = x(8);

% compute the gravitational force of the sun on the earth
res2=epx^2+epy^2;
res=sqrt(res2);
fes=P.G*P.me*P.ms/res2;
fesx=-fes*epx/res;
fesy=-fes*epy/res;

% compute the gravitational force of the sun on the asteroid
ras2=apx^2+apy^2;
ras=sqrt(ras2);
fas=P.G*P.ma*P.ms/ras2;
fasx=-fas*apx/ras;
fasy=-fas*apy/ras;

% compute the gravitational force the earth on the asteroid
Dx=apx-epx;
Dy=apy-epy;
rae2 =Dx^2+Dy^2;
rae=sqrt(rae2);
fae=P.G*P.me*P.ma/rae2;
faex=-fae*Dx/rae;
faey=-fae*Dy/rae;

% compute the derivative of states associated with earth
pevx=(fesx-faex)/P.me;
pevy=(fesy-faey)/P.me;
pepx=evx;
pepy=evy;

% compute the derivative of states associated with asteroid
pavx=(fasx+faex)/P.ma;
pavy=(fasy+faey)/P.ma;
papx=avx;
papy=avy;

% pack the state derivatives
px=[pevx pevy pepx pepy pavx pavy papx papy]';
    
end