% This script sets parameter and simulates the earth and an asteroid
%
% Written by S.D. Sudhoff 
%   School of Electrical and Computer Engineering
%   1285 Electrical Engineering Building
%   West Lafayette, IN 47907-1285
%   Phone: 765-494-3246
%   Fax: 765-494-0676
%   E-mail: sudhoff@ecn.purdue.edu

% model parameters
P.ms    = 1.989e30;     % mass of sun (kg)
P.me    = 5.972e24;     % mass of earth (kg)
P.ma    = P.me*1e-7;    % mass of asteroid (kg)
P.G     = 6.674e-11;    % gravitational constant (N*(m/kg)^2)

% initial earth position based on t=0 being perilon (Jan 3, 2018)
P.evx0  = 0;            % x-component of initial earth velocity(m/s)
P.evy0  = 30300;        % y-component of initial earth velocity(m/s)
P.epx0  = 147.1e9;      % x-component of initial earth position(m)
P.epy0  = 0;            % y-component of initial earth position(m)

P.avx0  = -10000;       % x-component of initial asteroid velocity(m/s)
P.avy0  = 0;            % y-component of initial asteroid velocity(m/s)
P.apx0  = 1.45e+11;     % x-component of initial asteroid position(m)
P.apy0  = 1.76e+11;     % y-component of initial asteroid position(m)

% handy numbers
au=149597870700;        % astronomical unit, m
re=6.378e6;             % equatorial radius of earth 
hr=12e3;                % height of top of troposphere, m
hs=50e3;                % height of top of stratosphere, m
hm=80e3;                % height of top of mesosphere, m
ht=700e3;               % height of top of thermosphere, m
he=10000e3;             % height of top of exosphere, m
ra=re*(P.ma/P.me)^(1/3);% radius of asteroid assuming earth density, m
tday=24*60*60;          % length of a day in seconds
thour=24*60;            % length of hour in seconds
tmin=60;                % lenght of minute in seconds

% time values where we want the answer 
t0=0;
tf=2*365*tday;
tvals=linspace(t0,tf,round((tf-t0)/tday));

% maximum time step
SP.maxre=1e-20;         % relative error (s)
SP.maxae=0;             % absolute error (s)
SP.maxit=20;            % maximum interations

% peform the initial simulation
SP.maxt=thour;          % maximum time step (s)
xic=[P.evx0 P.evy0 P.epx0 P.epy0 P.avx0 P.avy0 P.apx0 P.apy0];
[t,x]=odetpc(@earth_asteroid_model,tvals,xic,P,SP);

% grab the waveforms
evx = x(:,1);
evy = x(:,2);
epx = x(:,3);
epy = x(:,4);
avx = x(:,5);
avy = x(:,6);
apx = x(:,7); 
apy = x(:,8);

% compute the distance and earth-asteroid clearance
d=sqrt((epx-apx).^2+(epy-apy).^2);    % center-to-center distance (m)
c=d-ra-re;                            % surface-to-surface clearance (m)
[dmin,i]=min(d);                      % find minimum distance, and index
                                      % of minimum distance (m)
vrmin=sqrt((avx(i)-evx(i))^2+(avy(i)-evy(i))^2);  % velocity of closest
                                                  % approach (m/s)
va=sqrt(avx.^2+avy.^2);               % asteroid velecity relative to sun 
                                      % (m/s)

disp(['Minimum clearance (km) = ' num2str(c(i)/1000)]);
disp(['Relative velocity at closest approach (km/s) = ' ...
      num2str(vrmin/1000)]);

% plot the orbits
figure(1)
plot(epx/au,epy/au,'b',apx/au,apy/au,'g',[0],[0],'ro');
xlabel('x, au');
ylabel('y, au');
grid on
title('Earth-Asteroid Trajectory');
legend('earth','asteroid','sun');
grid on

% plot the clearance
figure(2)
o=ones(size(t))/1e3;
td=t/(60*60*24);
semilogy(td,(c+re)/1e3,...
         td,re*o,td,(hr+re)*o,td,(hs+re)*o,td,(hm+re)*o, ...
         td,(ht+re)*o,td,(he+re)*o);
grid on
xlabel('t, days from Perilon');
ylabel('c, earth center - asteroid surface clearance, km')
legend('Asteroid near surface', ...
       'Earth Surface', ...
       'Troposphere', ...
       'Stratosphere', ...
       'Mesosphere', ...
       'Thermosphere', ...
       'Exosphere');
title('Center-to-Surface Earth-Asteroid Distance');

% plot the asteroid velocity relative to the sun
figure(3)
plot(td,va/1e3)
xlabel('t, days from Perilon');
ylabel('v_a, km/s');
grid on;
title('Asteroid Velocity (Relative to Sun)');